@@ -1,6 +1,12 @@
-from typing import Any
-
-from starlite.datastructures import File, Redirect, State, Stream, Template
+from starlite.datastructures import (
+    Cookie,
+    File,
+    Redirect,
+    ResponseHeader,
+    State,
+    Stream,
+    Template,
+)
 
 from .app import Starlite
 from .config import (
@@ -56,7 +62,7 @@ from .provide import Provide
 from .response import Response
 from .router import Router
 from .routes import BaseRoute, HTTPRoute, WebSocketRoute
-from .types import MiddlewareProtocol, Partial, ResponseHeader
+from .types import MiddlewareProtocol, Partial
 
 __all__ = [
     "ASGIRouteHandler",
@@ -67,11 +73,12 @@ __all__ = [
     "Body",
     "CORSConfig",
     "CacheConfig",
+    "CompressionConfig",
     "Controller",
+    "Cookie",
     "DTOFactory",
     "Dependency",
     "File",
-    "CompressionConfig",
     "HTTPException",
     "HTTPRoute",
     "HTTPRouteHandler",
@@ -121,26 +128,3 @@ __all__ = [
     "route",
     "websocket",
 ]
-
-
-_deprecated_imports = {"TestClient", "create_test_client", "create_test_request"}
-
-
-# pylint: disable=import-outside-toplevel
-def __getattr__(name: str) -> Any:
-    """Provide lazy importing as per https://peps.python.org/pep-0562/"""
-    if name not in _deprecated_imports:
-        raise AttributeError(f"Module {__package__} has no attribute {name}")
-
-    import warnings
-
-    warnings.warn(
-        f"Importing {name} from {__package__} is deprecated, use `from starlite.testing import {name}` instead",
-        DeprecationWarning,
-        stacklevel=2,
-    )
-
-    from . import testing
-
-    attr = globals()[name] = getattr(testing, name)
-    return attr
