@@ -12,7 +12,14 @@ from starlette.responses import StreamingResponse
 from starlette.status import HTTP_200_OK, HTTP_201_CREATED, HTTP_204_NO_CONTENT
 
 from starlite.constants import REDIRECT_STATUS_CODES
-from starlite.datastructures import File, Redirect, StarliteType, Stream, Template
+from starlite.datastructures import (
+    Cookie,
+    File,
+    Redirect,
+    ResponseHeader,
+    Stream,
+    Template,
+)
 from starlite.enums import HttpMethod, MediaType
 from starlite.exceptions import (
     HTTPException,
@@ -33,11 +40,12 @@ from starlite.types import (
     AfterResponseHandler,
     BeforeRequestHandler,
     CacheKeyBuilder,
+    Empty,
+    EmptyType,
     ExceptionHandler,
     Guard,
     Method,
     Middleware,
-    ResponseHeader,
 )
 from starlite.utils import is_async_callable
 
@@ -49,6 +57,12 @@ if TYPE_CHECKING:
 
 class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
     __slots__ = (
+        "_resolved_after_request",
+        "_resolved_after_response",
+        "_resolved_before_request",
+        "_resolved_response_class",
+        "_resolved_response_cookies",
+        "_resolved_response_headers",
         "after_request",
         "after_response",
         "background_tasks",
@@ -64,12 +78,8 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         "media_type",
         "operation_id",
         "raises",
-        "resolved_after_request",
-        "resolved_before_request",
-        "resolved_after_response",
-        "resolved_headers",
-        "resolved_response_class",
         "response_class",
+        "response_cookies",
         "response_description",
         "response_headers",
         "status_code",
@@ -98,6 +108,7 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         middleware: Optional[List[Middleware]] = None,
         opt: Optional[Dict[str, Any]] = None,
         response_class: Optional[Type[Response]] = None,
+        response_cookies: Optional[List[Cookie]] = None,
         response_headers: Optional[Dict[str, ResponseHeader]] = None,
         status_code: Optional[int] = None,
         sync_to_thread: bool = False,
@@ -145,6 +156,7 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         self.cache_key_builder = cache_key_builder
         self.media_type = media_type
         self.response_class = response_class
+        self.response_cookies = response_cookies
         self.response_headers = response_headers
         self.sync_to_thread = sync_to_thread
         # OpenAPI related attributes
@@ -158,25 +170,20 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         self.response_description = response_description
         self.summary = summary
         self.tags = tags
-        # memoized attributes, defaulted to BaseRouteHandler.empty
-        self.resolved_after_request: Union[
-            Optional[AfterRequestHook], Type[BaseRouteHandler.empty]
-        ] = BaseRouteHandler.empty
-        self.resolved_after_response: Union[
-            Optional[AfterResponseHook], Type[BaseRouteHandler.empty]
-        ] = BaseRouteHandler.empty
-        self.resolved_before_request: Union[
-            Optional[BeforeRequestHook], Type[BaseRouteHandler.empty]
-        ] = BaseRouteHandler.empty
-        self.resolved_headers: Union[Dict[str, ResponseHeader], Type[BaseRouteHandler.empty]] = BaseRouteHandler.empty
-        self.resolved_response_class: Union[Type[Response], Type[BaseRouteHandler.empty]] = BaseRouteHandler.empty
+        # memoized attributes, defaulted to Empty
+        self._resolved_after_request: Union[Optional[AfterRequestHook], EmptyType] = Empty
+        self._resolved_after_response: Union[Optional[AfterResponseHook], EmptyType] = Empty
+        self._resolved_before_request: Union[Optional[BeforeRequestHook], EmptyType] = Empty
+        self._resolved_response_headers: Union[Dict[str, ResponseHeader], EmptyType] = Empty
+        self._resolved_response_cookies: Union[List[Cookie], EmptyType] = Empty
+        self._resolved_response_class: Union[Type[Response], EmptyType] = Empty
 
     def __call__(self, fn: "AnyCallable") -> "HTTPRouteHandler":
         """
         Replaces a function with itself
         """
         self.fn = fn
-        self.validate_handler_function()
+        self._validate_handler_function()
         return self
 
     def resolve_response_class(self) -> Type[Response]:
@@ -185,12 +192,12 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
 
         This method is memoized so the computation occurs only once.
         """
-        if self.resolved_response_class is BaseRouteHandler.empty:
-            self.resolved_response_class = Response
+        if self._resolved_response_class is Empty:
+            self._resolved_response_class = Response
             for layer in self.ownership_layers:
                 if layer.response_class is not None:
-                    self.resolved_response_class = layer.response_class
-        return cast("Type[Response]", self.resolved_response_class)
+                    self._resolved_response_class = layer.response_class
+        return cast("Type[Response]", self._resolved_response_class)
 
     def resolve_response_headers(self) -> Dict[str, "ResponseHeader"]:
         """
@@ -198,11 +205,27 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
 
         This method is memoized so the computation occurs only once.
         """
-        if self.resolved_headers is BaseRouteHandler.empty:
-            self.resolved_headers = {}
+        if self._resolved_response_headers is Empty:
+            self._resolved_response_headers = {}
             for layer in self.ownership_layers:
-                self.resolved_headers.update(layer.response_headers or {})
-        return cast("Dict[str, ResponseHeader]", self.resolved_headers)
+                self._resolved_response_headers.update(layer.response_headers or {})
+        return cast("Dict[str, ResponseHeader]", self._resolved_response_headers)
+
+    def resolve_response_cookies(self) -> List[Cookie]:
+        """
+        Returns a list of Cookie instances. Filters the list to ensure each cookie key is unique.
+
+        This method is memoized so the computation occurs only once.
+        """
+        if self._resolved_response_cookies is Empty:
+            self._resolved_response_cookies = []
+            cookies = []
+            for layer in self.ownership_layers:
+                cookies.extend(layer.response_cookies or [])
+            for cookie in reversed(cookies):
+                if not any(cookie.key == c.key for c in self._resolved_response_cookies):
+                    self._resolved_response_cookies.append(cookie)
+        return cast("List[Cookie]", self._resolved_response_cookies)
 
     def resolve_before_request(self) -> Optional["BeforeRequestHook"]:
         """
@@ -211,9 +234,9 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         If a handler is found it is returned, otherwise None is set.
         This method is memoized so the computation occurs only once
         """
-        if self.resolved_before_request is BaseRouteHandler.empty:
-            self.resolved_before_request = BeforeRequestHook.resolve_for_handler(self, "before_request")
-        return cast("Optional[BeforeRequestHook]", self.resolved_before_request)
+        if self._resolved_before_request is Empty:
+            self._resolved_before_request = BeforeRequestHook.resolve_for_handler(self, "before_request")
+        return cast("Optional[BeforeRequestHook]", self._resolved_before_request)
 
     def resolve_after_request(self) -> Optional["AfterRequestHook"]:
         """
@@ -222,9 +245,9 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         If a handler is found it is returned, otherwise None is set.
         This method is memoized so the computation occurs only once
         """
-        if self.resolved_after_request is BaseRouteHandler.empty:
-            self.resolved_after_request = AfterRequestHook.resolve_for_handler(self, "after_request")
-        return cast("Optional[AfterRequestHook]", self.resolved_after_request)
+        if self._resolved_after_request is Empty:
+            self._resolved_after_request = AfterRequestHook.resolve_for_handler(self, "after_request")
+        return cast("Optional[AfterRequestHook]", self._resolved_after_request)
 
     def resolve_after_response(self) -> Optional["AfterResponseHook"]:
         """
@@ -233,9 +256,9 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         If a handler is found it is returned, otherwise None is set.
         This method is memoized so the computation occurs only once
         """
-        if self.resolved_after_response is BaseRouteHandler.empty:
-            self.resolved_after_response = AfterResponseHook.resolve_for_handler(self, "after_response")
-        return cast("Optional[AfterResponseHook]", self.resolved_after_response)
+        if self._resolved_after_response is Empty:
+            self._resolved_after_response = AfterResponseHook.resolve_for_handler(self, "after_response")
+        return cast("Optional[AfterResponseHook]", self._resolved_after_response)
 
     @property
     def http_methods(self) -> List["Method"]:
@@ -244,11 +267,54 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         """
         return cast("List[Method]", self.http_method if isinstance(self.http_method, list) else [self.http_method])
 
-    def validate_handler_function(self) -> None:
+    async def to_response(self, data: Any, app: "Starlite", plugins: List[PluginProtocol]) -> StarletteResponse:
+        """
+        Given a data kwarg, determine its type and return the appropriate response
+        """
+        if isawaitable(data):
+            data = await data
+        media_type = self.media_type.value if isinstance(self.media_type, Enum) else self.media_type
+        headers = {k: v.value for k, v in self.resolve_response_headers().items() if not v.documentation_only}
+        cookies = self.resolve_response_cookies()
+        response: StarletteResponse
+        if isinstance(data, (StarletteResponse, Redirect, File, Stream, Template)):
+            response = self._get_response_from_data(
+                app=app,
+                cookies=cookies,
+                data=data,
+                headers=headers,
+                media_type=media_type,
+            )
+        else:
+            plugin = get_plugin_for_value(value=data, plugins=plugins)
+            if plugin:
+                if is_async_callable(plugin.to_dict):
+                    if isinstance(data, (list, tuple)):
+                        data = [await plugin.to_dict(datum) for datum in data]  # type: ignore
+                    else:
+                        data = await plugin.to_dict(data)  # type: ignore
+                else:
+                    if isinstance(data, (list, tuple)):
+                        data = [plugin.to_dict(datum) for datum in data]
+                    else:
+                        data = plugin.to_dict(data)
+            response_class = self.resolve_response_class()
+            response = response_class(
+                background=self.background_tasks,
+                content=data,
+                headers=headers,
+                media_type=media_type,
+                status_code=self.status_code,
+            )
+            for cookie in self._normalize_cookies(cookies, []):
+                response.set_cookie(**cookie)
+        return await self._process_after_request_hook(response)
+
+    def _validate_handler_function(self) -> None:
         """
         Validates the route handler function once it is set by inspecting its return annotations
         """
-        super().validate_handler_function()
+        super()._validate_handler_function()
         signature = Signature.from_callable(cast("AnyCallable", self.fn))
         return_annotation = signature.return_annotation
         if return_annotation is Signature.empty:
@@ -270,35 +336,79 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
         if "data" in signature.parameters and "GET" in self.http_methods:
             raise ImproperlyConfiguredException("'data' kwarg is unsupported for 'GET' request handlers")
 
+    @staticmethod
+    def _normalize_cookies(local_cookies: List[Cookie], layered_cookies: List[Cookie]) -> List[Dict[str, Any]]:
+        """
+        Given two lists of cookies, ensures the uniqueness of cookies by key
+        and returns a normalized dict ready to be set on the response.
+        """
+        filtered_cookies = [*local_cookies]
+        for cookie in layered_cookies:
+            if not any(cookie.key == c.key for c in filtered_cookies):
+                filtered_cookies.append(cookie)
+        normalized_cookies: List[Dict[str, Any]] = []
+        for cookie in filtered_cookies:
+            if not cookie.documentation_only:
+                normalized_cookies.append(cookie.dict(exclude_none=True, exclude={"documentation_only", "description"}))
+        return normalized_cookies
+
     def _get_response_from_data(
         self,
-        headers: dict,
-        data: Union[StarletteResponse, StarliteType],
-        media_type: Union[MediaType, str],
         app: "Starlite",
+        cookies: List[Cookie],
+        data: Union[StarletteResponse, Redirect, File, Stream, Template],
+        headers: dict,
+        media_type: Union[MediaType, str],
     ) -> StarletteResponse:
         """
         Determines the correct response type to return given data
         """
-        if isinstance(data, Redirect):
-            return RedirectResponse(headers=headers, status_code=self.status_code, url=data.path)
-        if isinstance(data, File):
-            return FileResponse(media_type=media_type, headers=headers, **data.dict())
-        if isinstance(data, Stream):
-            return StreamingResponse(
-                content=data.iterator, status_code=self.status_code, media_type=media_type, headers=headers
-            )
-        if isinstance(data, Template):
-            if not app.template_engine:
-                raise ImproperlyConfiguredException("Template engine is not configured")
-            return TemplateResponse(
-                context=data.context,
-                template_name=data.name,
-                template_engine=app.template_engine,
-                status_code=self.status_code,
-                headers=headers,
-            )
-        return cast("StarletteResponse", data)
+        if isinstance(data, (Redirect, File, Stream, Template)):
+            headers.update(data.headers)
+            normalized_cookies = self._normalize_cookies(data.cookies, cookies)
+            if isinstance(data, Redirect):
+                response: "StarletteResponse" = RedirectResponse(
+                    headers=headers, status_code=self.status_code, url=data.path, background=data.background
+                )
+            elif isinstance(data, File):
+                response = FileResponse(
+                    background=data.background,
+                    filename=data.filename,
+                    headers=headers,
+                    media_type=media_type,
+                    path=data.path,
+                    stat_result=data.stat_result,
+                )
+            elif isinstance(data, Stream):
+                response = StreamingResponse(
+                    background=data.background,
+                    content=data.iterator,
+                    headers=headers,
+                    media_type=media_type,
+                    status_code=self.status_code,
+                )
+            else:
+                if not app.template_engine:
+                    raise ImproperlyConfiguredException("Template engine is not configured")
+                response = TemplateResponse(
+                    background=data.background,
+                    context=data.context,
+                    headers=headers,
+                    status_code=self.status_code,
+                    template_engine=app.template_engine,
+                    template_name=data.name,
+                )
+        elif isinstance(data, Response):
+            response = data
+            normalized_cookies = self._normalize_cookies(data.cookies, cookies)
+        else:
+            response = data
+            normalized_cookies = self._normalize_cookies(cookies, [])
+
+        for cookie in normalized_cookies:
+            response.set_cookie(**cookie)
+
+        return response
 
     async def _process_after_request_hook(
         self,
@@ -312,40 +422,6 @@ class HTTPRouteHandler(BaseRouteHandler["HTTPRouteHandler"]):
             return await after_request(response)
         return response
 
-    async def to_response(self, data: Any, app: "Starlite", plugins: List[PluginProtocol]) -> StarletteResponse:
-        """
-        Given a data kwarg, determine its type and return the appropriate response
-        """
-        if isawaitable(data):
-            data = await data
-        media_type = self.media_type.value if isinstance(self.media_type, Enum) else self.media_type
-        headers = {k: v.value for k, v in self.resolve_response_headers().items()}
-        response: StarletteResponse
-        if isinstance(data, (StarletteResponse, StarliteType)):
-            response = self._get_response_from_data(headers=headers, data=data, media_type=media_type, app=app)
-        else:
-            plugin = get_plugin_for_value(value=data, plugins=plugins)
-            if plugin:
-                if is_async_callable(plugin.to_dict):
-                    if isinstance(data, (list, tuple)):
-                        data = [await plugin.to_dict(datum) for datum in data]  # type: ignore
-                    else:
-                        data = await plugin.to_dict(data)  # type: ignore
-                else:
-                    if isinstance(data, (list, tuple)):
-                        data = [plugin.to_dict(datum) for datum in data]
-                    else:
-                        data = plugin.to_dict(data)
-            response_class = self.resolve_response_class()
-            response = response_class(
-                headers=headers,
-                status_code=self.status_code,
-                content=data,
-                media_type=media_type,
-                background=self.background_tasks,
-            )
-        return await self._process_after_request_hook(response)
-
 
 route = HTTPRouteHandler
 
@@ -368,6 +444,7 @@ class get(HTTPRouteHandler):
         middleware: Optional[List[Middleware]] = None,
         opt: Optional[Dict[str, Any]] = None,
         response_class: Optional[Type[Response]] = None,
+        response_cookies: Optional[List[Cookie]] = None,
         response_headers: Optional[Dict[str, ResponseHeader]] = None,
         status_code: Optional[int] = None,
         sync_to_thread: bool = False,
@@ -406,6 +483,7 @@ class get(HTTPRouteHandler):
             raises=raises,
             response_class=response_class,
             response_description=response_description,
+            response_cookies=response_cookies,
             response_headers=response_headers,
             status_code=status_code,
             summary=summary,
@@ -432,6 +510,7 @@ class post(HTTPRouteHandler):
         middleware: Optional[List[Middleware]] = None,
         opt: Optional[Dict[str, Any]] = None,
         response_class: Optional[Type[Response]] = None,
+        response_cookies: Optional[List[Cookie]] = None,
         response_headers: Optional[Dict[str, ResponseHeader]] = None,
         status_code: Optional[int] = None,
         sync_to_thread: bool = False,
@@ -470,6 +549,7 @@ class post(HTTPRouteHandler):
             raises=raises,
             response_class=response_class,
             response_description=response_description,
+            response_cookies=response_cookies,
             response_headers=response_headers,
             status_code=status_code,
             summary=summary,
@@ -496,6 +576,7 @@ class put(HTTPRouteHandler):
         middleware: Optional[List[Middleware]] = None,
         opt: Optional[Dict[str, Any]] = None,
         response_class: Optional[Type[Response]] = None,
+        response_cookies: Optional[List[Cookie]] = None,
         response_headers: Optional[Dict[str, ResponseHeader]] = None,
         status_code: Optional[int] = None,
         sync_to_thread: bool = False,
@@ -534,6 +615,7 @@ class put(HTTPRouteHandler):
             raises=raises,
             response_class=response_class,
             response_description=response_description,
+            response_cookies=response_cookies,
             response_headers=response_headers,
             status_code=status_code,
             summary=summary,
@@ -560,6 +642,7 @@ class patch(HTTPRouteHandler):
         middleware: Optional[List[Middleware]] = None,
         opt: Optional[Dict[str, Any]] = None,
         response_class: Optional[Type[Response]] = None,
+        response_cookies: Optional[List[Cookie]] = None,
         response_headers: Optional[Dict[str, ResponseHeader]] = None,
         status_code: Optional[int] = None,
         sync_to_thread: bool = False,
@@ -598,6 +681,7 @@ class patch(HTTPRouteHandler):
             raises=raises,
             response_class=response_class,
             response_description=response_description,
+            response_cookies=response_cookies,
             response_headers=response_headers,
             status_code=status_code,
             summary=summary,
@@ -624,6 +708,7 @@ class delete(HTTPRouteHandler):
         middleware: Optional[List[Middleware]] = None,
         opt: Optional[Dict[str, Any]] = None,
         response_class: Optional[Type[Response]] = None,
+        response_cookies: Optional[List[Cookie]] = None,
         response_headers: Optional[Dict[str, ResponseHeader]] = None,
         status_code: Optional[int] = None,
         sync_to_thread: bool = False,
@@ -662,6 +747,7 @@ class delete(HTTPRouteHandler):
             raises=raises,
             response_class=response_class,
             response_description=response_description,
+            response_cookies=response_cookies,
             response_headers=response_headers,
             status_code=status_code,
             summary=summary,
